-- hds header_start
--
-- VHDL Architecture CALICE_seriface.protocol.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 21:33:41 11/02/03
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity protocol is
   port( 
      clk         : in     std_logic;
      rst         : in     std_logic;
      sdata_in    : in     std_logic_vector (7 downto 0);
      sdin_wren   : in     std_logic;
      addr_o      : out    std_logic_vector (15 downto 0);
      addr_strb_o : out    std_logic;
      db_rd_o     : out    std_logic;
      db_wr_o     : out    std_logic;
      sdata_out   : out    std_logic_vector (7 downto 0);
      sdout_rdy   : out    std_logic;
      db_io       : inout  std_logic_vector (31 downto 0);
      txrdy_i     : in     std_logic;
      trigger_o   : out    std_logic
   );

-- Declarations

end protocol ;

-- hds interface_end


-- The serial protocol
-- 7 bytes out, 5 bytes back ALWAYS.
-- out   = cmd,  addr1, addr0, data3, data2, data1, data0
-- reply = cmd*, data3, data2, data1, data0

--* has reply bit set 

architecture rtl of protocol is

	subtype  byte is std_logic_vector(7 downto 0);
	type     msgi_array 		is array ( 1 to  8) of byte;
	type     msgo_array 		is array (11 to 18) of byte;
	
	signal   msgi_buff		: msgi_array;
	signal   msgo_buff		: msgo_array;

	signal   msgi_dest		: byte;
	signal   msgo_dest		: byte;
	
	signal   msgi_cmd  		: byte;
	signal   msgo_cmd			: byte;

	signal   msgi_addr		: std_logic_vector(15 downto 0);
	signal   msgo_addr		: std_logic_vector(15 downto 0);

	signal   msgi_data		: std_logic_vector(31 downto 0);
	signal   msgo_data		: std_logic_vector(31 downto 0);
	

	-- Time Out
	signal   timeout  		: std_logic;
	signal   timeout_rst		: std_logic;
	
	-- Data Bus
	signal   db_out_en 		: std_logic;
	signal   db_in_en 		: std_logic;
	signal 	sdout_rdy_done : std_logic;
	

	-- Commands
	signal   cmd_all_bits   : std_logic_vector(8 downto 0);
	signal 	cmd_test  		: std_logic;
   signal 	cmd_write 		: std_logic;
   signal 	cmd_read  		: std_logic;
   signal 	cmd_readro 		: std_logic;
   signal 	cmd_stat  		: std_logic;
   signal 	cmd_trig  		: std_logic;   


	-- Constants
	constant TEST_REPLY		: std_logic_vector(31 downto 0) := 
		"10100001000100001010000100010000"; -- 0xAll0All0

	constant TRIG_REPLY		: std_logic_vector(31 downto 0) := 
		"11101110111011101010101010101010"; -- send 0xEEEEAAAA

	constant INVALID_REPLY	: std_logic_vector(31 downto 0) :=  	
		"11011110101011011010000011010000"; -- 0xDEADD0D0


	
begin

	---------------------------------------------------------------------------
	-- Message Buffer Signals Map
	---------------------------------------------------------------------------	

	-- Incoming Message
	-------------------
	msgi_dest  	 <= msgi_buff(1);
	msgi_cmd	  	 <= msgi_buff(2);

	msgi_addr( 7 downto  0) <= msgi_buff(3);
	msgi_addr(15 downto  8) <= msgi_buff(4);
	
	msgi_data( 7 downto  0) <= msgi_buff(5);
	msgi_data(15 downto  8) <= msgi_buff(6);
	msgi_data(23 downto 16) <= msgi_buff(7);
	msgi_data(31 downto 24) <= msgi_buff(8);

	
	-- Outgoing Message
	-------------------
	msgo_buff(11) <= msgo_dest;
	msgo_buff(12) <= msgo_cmd;
	
	msgo_buff(13) <= msgo_addr( 7 downto  0);
	msgo_buff(14) <= msgo_addr(15 downto  8);
	
	msgo_buff(15) <= msgo_data( 7 downto  0);
	msgo_buff(16) <= msgo_data(15 downto  8);
	msgo_buff(17) <= msgo_data(23 downto 16);
	msgo_buff(18) <= msgo_data(31 downto 24);

	
	
	
	
	---------------------------------------------------------------------------
	-- Outgoing Message Generate
	---------------------------------------------------------------------------	


	msgo_dest  <= msgi_dest;
	msgo_cmd   <= msgi_cmd OR "00000001";
	
	msgo_addr <= msgi_addr;
		

	msgo_data_assign : process (clk)
	begin
		if rising_edge(clk) then

			if (cmd_test = '1') then -- send test data 0xAll0All0 back
				msgo_data <= TEST_REPLY;
											
			elsif (cmd_write = '1') then
				msgo_data <= msgi_data;
	
			elsif (cmd_read = '1') or (cmd_readro = '1') then
				if (db_in_en = '1') then
					msgo_data <= db_io; 	
				end if;

			elsif (cmd_trig = '1') then
				msgo_data <= TRIG_REPLY;
				
			else
				msgo_data <= INVALID_REPLY;
				
	
			end if;

		end if;
	end process;
	

	---------------------------------------------------------------------------
	-- Timeout Counter
	---------------------------------------------------------------------------
	timeout_counter : process (clk)
		variable timeout_cntr: integer range 0 to 32767; -- 15 bit	
	
	begin
		if rising_edge(clk) then
			if (rst = '1') or (timeout_rst = '1') then
				timeout_cntr := 0;
				timeout <= '0';
				
			else
		
				if (timeout_cntr = 32767) then
					timeout_cntr := 0;
					timeout <= '1';
										
				else
					timeout_cntr := timeout_cntr + 1;
					timeout <= '0';
					
				end if;
			end if;
		end if;
	end process;	
	
	

	---------------------------------------------------------------------------
	-- Serial Machine
	---------------------------------------------------------------------------
	machine1 : process (clk)
		variable state : integer range 0 to 31;	
	
	begin
		if rising_edge(clk) then
			if (rst = '1') or (timeout = '1') then
				state := 0;

			else
				-- defaults
				sdout_rdy   <= '0';
				db_in_en    <= '0';
				db_out_en   <= '0';
				timeout_rst <= '1';

				-- reset condition 
				if (state = 0) then
					addr_strb_o 	<= '0';
					db_in_en	 		<= '0';
					db_out_en 		<= '0';
					sdout_rdy_done <= '0';
										
				
					state := state + 1;
				
								
				-- wait for start of data *** this is messy 
				elsif (state = 1) then
				
					if (sdin_wren = '1') then  -- this should generate clock enables
						msgi_buff(state) <= sdata_in;
						
						state := state + 1;
					end if;


				-- receive external data
				elsif (state < 9) then
					timeout_rst <= '0';
				
					if (sdin_wren = '1') then  -- this should generate clock enables
						msgi_buff(state) <= sdata_in;
						timeout_rst <= '1';
						
						state := state + 1;
					end if;
					
				

				-- read/write strobes -- 2 clocks wide for safety ...
				elsif (state < 11) then

					if (cmd_read = '1')  then  db_in_en  <= '1'; end if;
					if (cmd_write = '1') then  db_out_en <= '1'; end if;
					
					state := state + 1;
				
					
				-- send external data				
				elsif (state < 19) then

					sdata_out <= msgo_buff(state);
					
					-- make sure only sent again after getting rdy
					if (sdout_rdy_done = '0') then
						sdout_rdy      <= '1';
						sdout_rdy_done <= '1';
					
					else
						sdout_rdy      <= '0';
							
						if (txrdy_i = '1') then -- wait for txrdy
							sdout_rdy_done <= '0';
							state := state + 1;
					
				   	end if;
					end if;

				
				-- back to zero								
				else
					state := 0;
					
				end if;


				-- generate data address strobe (it should remain high until reset).
				if (state = 3) then
					addr_strb_o <= '1';
				end if;
				
			end if;
		end if;
	end process;


	---------------------------------------------------------------------------
	-- Command Decoder
	---------------------------------------------------------------------------

	command_decoder : process (msgi_cmd)
   begin
    	
      case msgi_cmd is
         when "00000000" => cmd_all_bits <= "000000001"; -- 0x00 - test
         when "00000010" => cmd_all_bits <= "000000010"; -- 0x02 - write
         when "00000100" => cmd_all_bits <= "000000100"; -- 0x04 - read
         when "00000110" => cmd_all_bits <= "000001000"; -- 0x06 - readro
         when "00001000" => cmd_all_bits <= "000010000"; -- 0x08 - status
         -- more here (reset?)
         when "00010000" => cmd_all_bits <= "100000000"; -- 0x10 - trigger

         when others 	 => cmd_all_bits <= "000000000"; -- none of the above


      end case;
	end process;

	cmd_test   <= cmd_all_bits(0);
  	cmd_write  <= cmd_all_bits(1);
  	cmd_read   <= cmd_all_bits(2);
  	cmd_readro <= cmd_all_bits(3);
  	cmd_stat   <= cmd_all_bits(4);
  	cmd_trig   <= cmd_all_bits(8);


	trigger_o <= cmd_trig;




	---------------------------------------------------------------------------
	-- A16 D32 Bus Signals Map
	---------------------------------------------------------------------------

	-- Address Bus
	addr_o(15 downto 2) <= msgi_addr(15 downto  2);
	addr_o( 1 downto 0) <= "00";
	

	-- Data Bus
	db_rd_o <= db_in_en;
	db_wr_o <= db_out_en;
	
	db_io   <= msgi_data when (db_out_en = '1') else (others => 'Z');



end rtl;
